@@ -6,6 +6,8 @@ from scipy.signal import find_peaks
 import nifty_ls  # noqa: F401
 from scipy.stats import f as ftest
 
+import logging
+
 
 def alias_func(x, i, j, p_feat):
     """
@@ -122,8 +124,8 @@ def get_period_estimate(residuals_dataframe, p_min=0.03, p_max=2):
     )
 
 
-def get_multiband_period_estimate(
-    residuals_dataframe, p_min=0.03, p_max=2, k_free=True, k_val=None
+def get_multiterm_period_estimate(
+    residuals_dataframe, p_min=0.03, p_max=2, k_free=True, k_val=None, k_max=4
 ):
     """
     Estimate the period of a multiband time series using a multiband Lomb-Scargle model.
@@ -148,7 +150,8 @@ def get_multiband_period_estimate(
         If True, automatically scan multiple base-term complexities to choose optimal model. Default True.
     k_val : int, optional
         Fixed number of base terms to use if `k_free=False`. Default None.
-
+    k_max : int
+        Maximum number of terms to use for the multiterm LS periodogram. Default 4
     Returns
     -------
     tuple
@@ -175,7 +178,7 @@ def get_multiband_period_estimate(
     residuals = np.zeros(len(residuals_dataframe["filters"].values))
     bands = np.unique(residuals_dataframe["filters"].values)
     if k_free:
-        for k in range(1, 11):
+        for k in range(1, k_max + 1):
             model = LombScargleMultiband(
                 residuals_dataframe["jd"].values,
                 residuals_dataframe["residuals"].values,
@@ -186,13 +189,17 @@ def get_multiband_period_estimate(
                 nterms_base=k,
                 nterms_band=1,
             )
-            frequency, power = model.autopower(
-                method="fast",
-                sb_method="fastnifty_chi2",
-                minimum_frequency=1 / period_range[1],
-                maximum_frequency=1 / period_range[0],
-                samples_per_peak=5,
-            )
+            try:
+                frequency, power = model.autopower(
+                    method="fast",
+                    sb_method="fastnifty_chi2",
+                    minimum_frequency=1 / period_range[1],
+                    maximum_frequency=1 / period_range[0],
+                    samples_per_peak=5,
+                )
+            except np.linalg.LinAlgError:
+                logging.warning("Singular matrix for k={}".format(k))
+                continue
 
             f_best = frequency[np.argmax(power)]
             y_model = model.model(
@@ -209,7 +216,8 @@ def get_multiband_period_estimate(
 
             rms = np.sqrt(np.mean(residuals**2))
             n_params = 2 * k + 1 + 3 * len(bands)
-            dof = len(residuals) - n_params
+            dof = n_params
+            N = len(residuals)
 
             results.append((k, f_best, rms, dof, n_params))
 
@@ -218,13 +226,13 @@ def get_multiband_period_estimate(
         for i in range(len(results) - 1):
             k, f_best, rss, dof, n_params = results[i]
             k_next, f_best_next, rss_next, dof_next, n_params_next = results[i + 1]
-            F = ((rss - rss_next) / (dof - dof_next)) / (rss_next / dof_next)
+            F = ((rss**2 / rss_next**2) - 1) * ((N - dof_next) / (dof_next - dof))
 
             # Here crit = Fstat value for which model_2 (more complex) is in fact better than model_1 (less complex)
             crit = ftest.ppf(
                 q=0.99,
                 dfn=n_params_next - n_params,
-                dfd=len(residuals) - n_params_next,
+                dfd=N - n_params_next,
             )
 
             model_comparison.loc[i, "k"] = k
@@ -232,22 +240,15 @@ def get_multiband_period_estimate(
             model_comparison.loc[i, "f_best"] = f_best
             model_comparison.loc[i, "Fstat"] = F
             model_comparison.loc[i, "alpha_crit"] = crit
-            model_comparison.loc[i, "rms"] = rms
+            model_comparison.loc[i, "rms"] = rss
 
         cond = model_comparison["Fstat"] > model_comparison["alpha_crit"]
         model_comparison = model_comparison[
             ~cond
         ]  # don't go for the more complex model
-        f_chosen = model_comparison[
-            model_comparison.k == model_comparison.k.min()
-        ].f_best[0]  # Simplest model
-        k_val = model_comparison[model_comparison.k == model_comparison.k.min()].k[
-            0
-        ]  # Simplest model
-        p_rms = model_comparison[model_comparison.k == model_comparison.k.min()].rms[
-            0
-        ]  # Simplest model
-
+        f_chosen = model_comparison.loc[model_comparison.k.idxmin()]["f_best"]
+        k_val = model_comparison.loc[model_comparison.k.idxmin()]["k"]
+        p_rms = model_comparison.loc[model_comparison.k.idxmin()]["rms"]
     if not k_free:
         model = LombScargleMultiband(
             residuals_dataframe["jd"].values,
@@ -280,7 +281,7 @@ def get_multiband_period_estimate(
         residuals_dataframe["sigma"].values,
         normalization="standard",
         fit_mean=True,
-        nterms_base=k_val,
+        nterms_base=int(k_val),
         nterms_band=1,
     )
     frequency_final, power_final = model_final.autopower(
@@ -321,7 +322,7 @@ def get_multiband_period_estimate(
     signal_peaks = frequency_final[pindex[hindex]]
     window_peaks = frequencyW[pindexW[hindexW]]
 
-    return period_in, k_val, p_rms, signal_peaks, window_peaks
+    return period_in, int(k_val), p_rms, signal_peaks, window_peaks
 
 
 def perform_residual_resampling(resid_df, p_min, p_max, k=1):
@@ -337,7 +338,7 @@ def perform_residual_resampling(resid_df, p_min, p_max, k=1):
     ----------
     resid_df : pandas.DataFrame
         DataFrame of residuals containing columns required for `get_period_estimate`
-        or `get_multiband_period_estimate`.
+        or `get_multiterm_period_estimate`.
     p_min : float
         Minimum period to search (days).
     p_max : float
@@ -356,7 +357,7 @@ def perform_residual_resampling(resid_df, p_min, p_max, k=1):
     Notes
     -----
     - Performs 25 bootstrap resamples by default.
-    - For k=1, uses `get_period_estimate`; for k>1, uses `get_multiband_period_estimate`.
+    - For k=1, uses `get_period_estimate`; for k>1, uses `get_multiterm_period_estimate`.
     """
 
     if k == 1:
@@ -373,15 +374,17 @@ def perform_residual_resampling(resid_df, p_min, p_max, k=1):
         cond = np.abs(Pog - Pbs) / Pog < 1e-2
         Nbs = np.sum(np.ones(25)[cond])
     if k > 1:
-        Pog, _, _ = 24 / get_multiband_period_estimate(
+        Ptmp, _, _, _, _ = get_multiterm_period_estimate(
             resid_df, p_min=p_min, p_max=p_max, k_free=False, k_val=k
         )
+        Pog = 24 / Ptmp
         Pbs = np.zeros(25)
         for n in range(25):
             BS_df = resid_df.sample(n=len(resid_df), replace=True)
-            Pbs[n], _, _ = 24 / get_multiband_period_estimate(
+            Ptmp, _, _, _, _ = get_multiterm_period_estimate(
                 BS_df, p_min=p_min, p_max=p_max, k_free=False, k_val=k
             )
+            Pbs[n] = 24 / Ptmp
         cond = np.abs(Pog - Pbs) / Pog < 1e-2
         Nbs = np.sum(np.ones(25)[cond])
     return BS_df, Nbs
