@@ -418,3 +418,77 @@ def generate_initial_points(ra, dec, dec_shift=45):
                 ra_list.append(wrap_longitude(temp_ra + offset))
                 dec_list.append(shifted_dec)
     return ra_list, dec_list
+
+
+def gaussian_interpolate(data, factor=4, sigma=1.0):
+    """
+    Reproduce matplotlib's `interpolation="gaussian"` effect.
+    factor : upsampling factor
+    sigma  : Gaussian smoothing strength
+    """
+    from scipy.ndimage import zoom, gaussian_filter
+
+    # Step 1: upsample (mimics interpolation grid)
+    up = zoom(data, factor, order=1)  # bilinear before smoothing
+
+    # Step 2: apply gaussian smoothing
+    smoothed = gaussian_filter(up, sigma=sigma)
+
+    return smoothed
+
+
+def detect_local_minima(arr):
+    import scipy.ndimage.filters as filters
+    import scipy.ndimage.morphology as morphology
+
+    # https://stackoverflow.com/questions/3684484/peak-detection-in-a-2d-array/3689710#3689710
+    # https://stackoverflow.com/questions/3986345/how-to-find-the-local-minima-of-a-smooth-multidimensional-array-in-numpy
+    """
+    Takes an array and detects the troughs using the local maximum filter.
+    Returns a boolean mask of the troughs (i.e. 1 when
+    the pixel's value is the neighborhood maximum, 0 otherwise)
+    """
+    neighborhood = morphology.generate_binary_structure(len(arr.shape), 2)
+
+    footprint = np.ones((15, 15))
+    local_min = filters.minimum_filter(arr, footprint=footprint) == arr
+
+    background = arr == 0
+    eroded_background = morphology.binary_erosion(
+        background, structure=neighborhood, border_value=1
+    )
+
+    detected_minima = local_min ^ eroded_background
+    return np.where(detected_minima)
+
+
+def trumpet(peak_diff_1, f_feat, f_obs):
+    """
+    Implementation of the alias/true flagging algorithm.
+
+    Parameters
+    ----------
+    peak_diff_1 : float
+        The difference between the 2nd and 3rd highest peaks of the periodogram
+    f_feat : float
+        The feature frequency.
+    f_obs : float
+        The 1st highest peak if the periodogram
+    Returns
+    -------
+    The assumed true frequency peak difference value if the peak is true,
+    otherwise zero.
+    """
+    if peak_diff_1 > 0 and f_obs > f_feat:
+        return 2 * f_feat
+
+    elif peak_diff_1 < 0 and f_obs < f_feat:
+        return -2 * f_obs
+
+    elif peak_diff_1 > 0 and f_obs < f_feat:
+        return 2 * f_obs
+
+    elif peak_diff_1 < 0 and f_obs > f_feat:
+        return -2 * f_feat
+
+    return 0.0
